package com.example.demo_OptionalFields;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo_OptionalFields.OptionalFieldsEntity.Names;

@SpringBootTest
class OptionalFieldsRepoTest {

	@Autowired
	private OptionalFieldsRepo testClass;

	private OptionalFieldsEntity buildData(String name, String usedName) {
		OptionalFieldsEntity user1 = new OptionalFieldsEntity();
		user1.setName(name);
		if (usedName != null) {
			user1.setUsedName(usedName);
		}
		return user1;
	}

	@Test
	void test_findAll_hql() {
		testClass.save(buildData("Mary", "Shuning"));
		testClass.save(buildData("Zheng", null));

		List<OptionalFieldsEntity> allData = testClass.findAll();
		assertEquals("Shuning", allData.get(0).getUsedName());
		assertNull(allData.get(1).getUsedName());
	}

	@Test
	void test_findAll_projection() {
		testClass.save(buildData("Mary", "Shuning"));
		testClass.save(buildData("Zheng", null));

		List<OptionalFieldsEntity.IdName> allData = testClass.finalAllIdName();
		assertEquals("Mary", allData.get(0).getName());
		assertEquals("Zheng", allData.get(1).getName());

		List<Names> names = testClass.finalAllNames();
		assertEquals("Mary", names.get(0).getName());
		assertEquals("Zheng", names.get(1).getName());
		assertEquals("Shuning", names.get(0).getUsedName());
		assertNull(names.get(1).getUsedName());
	}

	@Test
	void test_findAll_raw_sql() {
		testClass.save(buildData("Mary", "Shuning"));
		testClass.save(buildData("Zheng", null));

		List<OptionalFieldsEntity> allData = testClass.finalAllNamesWithoutUsedName();
		assertEquals(1, allData.size());
		assertEquals("Zheng", allData.get(0).getName());
		assertNull(allData.get(0).getUsedName());

	}

}
